#!/usr/bin/python
# -*- coding: utf-8 -*-

from epyk.core.js import JsUtils
from epyk.core.js.primitives import JsNumber
from epyk.core.js.primitives import JsObject
from epyk.core.js.packages import JsPackage

# All the predefined variable types
from epyk.core.js.fncs import JsFncs


class D3ScaleLinear:
  """

  """
  def __init__(self, range):
    self.range = range

  def range(self):
    pass


# class D3Html(JsPackage):
#   lib_alias = {"js": 'd3'}
#
#   def __init__(self, src, selector, tag, setVar=False):
#     self.src, self._selector, self.setVar, self.tag = src, selector, setVar, tag
#     self.__js_uniq_fncs, self.varName, self.setVar, self.src = {}, None, setVar, src
#     self._js, self._u, self._js_enums = [[]], {}, {} # a list of list of object definition
#
#   def append(self, htmlType, setVar=True):
#     """
#     If the specified type is a string, appends a new element of this type (tag name) as the last child of each selected element, or before the next following sibling in the update selection if this is an enter selection.
#
#     Documentation
#     https://github.com/d3/d3-selection/blob/v1.4.0/README.md#selecting-elements
#
#     :param htmlType:
#     """
#     cnv_htmlType = JsUtils.jsConvertData(htmlType, None)
#     return D3Html(self.src, "%s.append(%s)" % (self.toStr(), cnv_htmlType), htmlType, setVar=setVar)
#
#   def rappend(self, htmlType):
#     return self.append(htmlType, setVar=False)
#
#   def attr(self, key, val):
#     """
#
#     :param key:
#     :param val:
#     """
#     return self.fnc("attr(%s, %s)" % (JsUtils.jsConvertData(key, None), JsUtils.jsConvertData(val, None)))
#
#   def text(self, data):
#     return self.fnc("text(function(column) { return column; })")
#
#   def html(self, data):
#     return self.fnc("html(function(d) { return d.value; })")
#
#   def selectAll(self, d3Type):
#     self.fnc("selectAll(%s)" % JsUtils.jsConvertData(d3Type, None))
#     return D3Select(self.src, selector=self.toStr())


class D3Select(JsPackage):
  lib_alias = {"js": 'd3'}

  def data(self, datasets=None):
    """
    Description:
    -----------
    Binds the specified array of data with the selected elements,
    returning a new selection that represents the update selection: the elements successfully bound to data.

    The data is specified for each group in the selection.
    If the selection has multiple groups (such as d3.selectAll followed by selection.selectAll),
    then data should typically be specified as a function.

    Related Pages:

      https://github.com/d3/d3-selection/blob/v1.4.0/README.md#selection_data

    Attributes:
    ----------
    :param datasets: List. The data set object.
    """
    if datasets is None:
      self.fnc("data()")
    else:
      #datasets = JsUtils.jsConvertData(datasets, None)
      self.fnc("data(%s)" % datasets)
    return self

  def dataRecord(self, columns):
    """
    Description:
    -----------
    Convert a list of dictionaries to an iteractor which will return ordered arrays based on the columns definition

    Related Pages:

      http://bl.ocks.org/gka/17ee676dc59aa752b4e6

    Attributes:
    ----------
    :param columns: List. The columns to be retrieve for each row
    """
    return self.fnc('''data(function(row, i) { var cell = []; %s.forEach(function(k) { cell.push(row[k]);
        }); return cell; }) ''' % JsUtils.jsConvertData(columns, None))

  def dataFncRows(self, columns):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param columns:
    """
    self.fnc("data(function(row) {return %s.map(function(c, i) {return {column: c, value: row[i]}; }); })" % columns)
    return self

  def datum(self, datasets=None):
    """
    Description:
    -----------
    Gets or sets the bound data for each selected element.
    Unlike selection.data, this method does not compute a join and does not affect indexes or the enter and exit selections.

    Related Pages:

      https://github.com/d3/d3-selection/blob/v1.4.0/README.md#selection_datum

    :param datasets: If a value is not specified, returns the bound datum for the first (non-null) element in the selection.
    """
    if datasets is None:
      self.fnc("datum()")
    else:
      self.fnc("datum(%s)" % JsUtils.jsConvertData(datasets, None))
    return self

  def filter(self, jsFncs, profile=False):
    """
    Description:
    -----------
    You can filter a selection using D3’s .filter method.
    The first argument is a function which returns true if the element should be included.
    The filtered selection is returned by the filter method so you can continue chaining selection methods.

    Related Pages:

      https://www.d3indepth.com/selections/

    Attributes:
    ----------
    :param jsFncs: String | List. The Javascript events when the DatePicker selection changes.
    :param profile: Boolean. Optional. Set to true to get the profile for the function on the Javascript console.
    """
    return self.fnc("filter(function(d, i) {%s} )" % (
      JsUtils.jsConvertFncs(jsFncs, toStr=True, profile=profile)))

  def sort(self, jsFncs, profile=False):
    """
    Description:
    -----------

    Related Pages:

      https://www.d3indepth.com/selections/

    Attributes:
    ----------
    :param jsFncs: String | List. The Javascript events when the DatePicker selection changes.
    :param profile: Boolean. Optional. Set to true to get the profile for the function on the Javascript console.
    """
    return self.fnc("sort(function(a, b) {%s} )" % (
      JsUtils.jsConvertFncs(jsFncs, toStr=True, profile=profile)))

  def enter(self):
    """
    Description:
    -----------
    Returns the enter selection: placeholder nodes for each datum that had no corresponding DOM element in the selection.
    (The enter selection is empty for selections not returned by selection.data.)

    Related Pages:

      https://github.com/d3/d3-selection/blob/v1.4.0/README.md#selection_enter
    """
    return self.fnc("enter()")

  def exit(self):
    """
    Description:
    -----------
    Returns the exit selection: existing DOM elements in the selection for which no new datum was found.
    (The exit selection is empty for selections not returned by selection.data.)

    Related Pages:

      https://github.com/d3/d3-selection/blob/v1.4.0/README.md#selection_exit
    """
    return self.fnc("exit()")

  def append(self, htmlType, setVar=True):
    """
    Description:
    -----------
    If the specified type is a string, appends a new element of this type (tag name) as the last child of each
    selected element, or before the next following sibling in the update selection if this is an enter selection.

    Related Pages:

      https://github.com/d3/d3-selection/blob/v1.4.0/README.md#selecting-elements

    Attributes:
    ----------
    :param htmlType:
    :param setVar:
    """
    return self.fnc("append(%s)" % JsUtils.jsConvertData(htmlType, None))

  def rappend(self, htmlType):
    """
    Description:
    -----------
    Recursive append function without defining the variable on the javascript side.

    Attributes:
    ----------
    :param htmlType:
    """
    return self.append(htmlType, setVar=False)

  def insert(self, htmlType, id):
    pass

  def style(self, key, val=None, callback=None):
    """
    Description:
    -----------
    Update the style.

    Related Pages:

      https://www.d3indepth.com/selections/

    Attributes:
    ----------
    :param key: String. The attribute key.
    :param val: String. Optional. The attribute value.
    :param callback: String. Optional. A javascript callback function using d (data) and i (index).
    """
    if val is None and callback is None:
      raise Exception("Either val or callback must be defined")

    if callback is not None:
      return self.fnc("style(%s, function(d, i){return %s})" % (JsUtils.jsConvertData(key, None), callback))

    return self.fnc("style(%s, %s)" % (JsUtils.jsConvertData(key, None), JsUtils.jsConvertData(val, None)))

  def property(self, key, val=None, callback=None):
    """
    Description:
    -----------
    Update an element's property.

    Related Pages:

      https://www.d3indepth.com/selections/

    Attributes:
    ----------
    :param key: String. The attribute key.
    :param val: String. Optional. The attribute value.
    :param callback: String. Optional. A javascript callback function using d (data) and i (index).
    """
    if val is None and callback is None:
      raise Exception("Either val or callback must be defined")

    if callback is not None:
      return self.fnc("property(%s, function(d, i){return %s})" % (JsUtils.jsConvertData(key, None), callback))

    return self.fnc("property(%s, %s)" % (JsUtils.jsConvertData(key, None), JsUtils.jsConvertData(val, None)))

  def attr(self, key, val=None, callback=None):
    """
    Description:
    -----------
    Update an attribute.

    Related Pages:

      https://gramener.github.io/d3js-playbook/events.html
      https://www.d3indepth.com/selections/

    Attributes:
    ----------
    :param key: String. The attribute key.
    :param val: String. Optional. The attribute value.
    :param callback: String. Optional. A javascript callback function using d (data) and i (index).
    """
    if val is None and callback is None:
      raise Exception("Either val or callback must be defined")

    if callback is not None:
      return self.fnc("attr(%s, function(d, i){return %s})" % (JsUtils.jsConvertData(key, None), callback))

    return self.fnc("attr(%s, %s)" % (JsUtils.jsConvertData(key, None), JsUtils.jsConvertData(val, None)))

  def text(self, data=None, column=None):
    """
    Description:
    -----------
    Update the text content.

    Related Pages:

      https://www.d3indepth.com/selections/

    Attributes:
    ----------
    :param data: Value. Optional. A Javascript function or a fixed value.
    :param column: String. Optional. the column name in the record.
    """
    if data is None and column is None:
      return self.fnc("text(function(d) { return d })")

    if column is not None:
      return self.fnc("text(function(d) { return d[%s]; })" % JsUtils.jsConvertData(column, None))

    return self.fnc("text(%s)" % JsUtils.jsConvertData(data, None))

  def transition(self, duration=None):
    """
    Description:
    -----------

    Related Pages:

      https://gramener.github.io/d3js-playbook/events.html

    Attributes:
    ----------
    :param duration: Integer. Optional. The time in ms used for the transition state.
    """
    if duration is None:
      return self.fnc("transition()")

    return self.fnc("transition().duration(%s)" % duration)

  def html(self, jsData=None):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param jsData:
    """
    if jsData is None:
      return self.fnc("html(function(d) { return d; })")

    return self.fnc("html(function(d) { return %s; })" % jsData)

  def htmlByKey(self, data):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param data:
    """
    return self.fnc("html(function(d) {console.log('rrr'+ d); return d[%s]; })" % JsUtils.jsConvertData(data, None))

  def call(self, fnc_name):
    """
    Description:
    -----------
    The .call method allows a function to be called into which the selection itself is passed as the first argument.

    Attributes:
    ----------
    :param fnc_name: String. A function name.
    """
    return self.fnc("call(%s)" % fnc_name)

  def on(self, eventType, jsFncs, profile=False):
    """
    Description:
    -----------
    Add an event to the component.

    Attributes:
    ----------
    :param eventType: String. The event name.
    :param jsFncs: String | List. The Javascript events when the DatePicker selection changes.
    :param profile: Boolean. Optional. Set to true to get the profile for the function on the Javascript console.
    """
    self.style("cursor", "pointer")
    return self.fnc("on(%s, function(data) {%s} )" % (
      JsUtils.jsConvertData(eventType, None), JsUtils.jsConvertFncs(jsFncs, toStr=True, profile=profile)))

  def hover(self, jsFncs, profile=False):
    """
    Description:
    -----------
    Mouse pointer has moved onto the element or its children.

    Related Pages:

      https://gramener.github.io/d3js-playbook/events.html
      https://www.d3indepth.com/selections/

    Attributes:
    ----------
    :param jsFncs: String | List. The Javascript events when the DatePicker selection changes.
    :param profile: Boolean. Optional. Set to true to get the profile for the function on the Javascript console.
    """
    return self.on("mouseover", jsFncs, profile)

  def click(self, jsFncs, profile=False):
    """
    Description:
    -----------
    Element has been clicked.

    Related Pages:

      https://gramener.github.io/d3js-playbook/events.html
      https://www.d3indepth.com/selections/

    Attributes:
    ----------
    :param jsFncs: String | List. The Javascript events when the DatePicker selection changes.
    :param profile: Boolean. Optional. Set to true to get the profile for the function on the Javascript console.
    """
    return self.on("click", jsFncs, profile)

  def remove(self):
    """
    Description:
    -----------
    Removes the selected elements from the document.
    Returns this selection (the removed elements) which are now detached from the DOM.
    There is not currently a dedicated API to add removed elements back to the document; however, you can pass a
    function to selection.append or selection.insert to re-add elements.

    Related Pages:

      https://github.com/d3/d3-selection/blob/v1.4.0/README.md#selecting-elements
    """
    return self.fnc("remove()")

  def clone(self, deep=False):
    """
    Description:
    -----------
    Inserts clones of the selected elements immediately following the selected elements and returns a selection of
    the newly added clones.

    Related Pages:

      https://github.com/d3/d3-selection/blob/v1.4.0/README.md#selecting-elements

    Attributes:
    ----------
    :param deep: Boolean. Optional. If deep is truthy, the descendant nodes of the selected elements will be cloned as well
    """
    return self.fnc("clone(%s)" % JsUtils.jsConvertData(deep, None))

  def order(self):
    """
    Description:
    -----------
    Re-inserts elements into the document such that the document order of each group matches the selection order.
    This is equivalent to calling selection.sort if the data is already sorted, but much faster.

    Related Pages:

      https://github.com/d3/d3-selection/blob/v1.4.0/README.md#selection_append
    """
    return self.fnc("order()")
  #
  # def select(self, id):
  #   return 'd3.select("%s")' % id

  def selectAll(self, d3Type):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param d3Type:
    """
    return self.fnc("selectAll(%s)" % JsUtils.jsConvertData(d3Type, None))


class D3ForceSimulation:
  _package = ['d3-force']

  def __init__(self, id=None, d3Type=None):
    """

    Documentation:
      - https://github.com/d3/d3-force

    :param id:
    :param d3Type:
    """
    pass

  def force(self, forceType, fnc):
    pass

  def restart(self):
    pass

  def stop(self):
    pass

  def tick(self, tick):
    pass

  def nodes(self, nodes):
    pass

  def alpha(self, number):
    pass

  def alphaDecay(self, number):
    pass

  def alphaTarget(self, target):
    pass

  def alphaTarget(self, number):
    pass

  def velocityDecay(self, decay):
    pass

  def find(self):
    pass

  def on(self):
    pass


class D3ForceManyBody:
  def strength(self):
    pass

  def distanceMax(self, x):
    pass

  def distanceMin(self, x):
    pass


class D3ForceCollide:
  def __init__(self, radius=None):
    """
    Creates a new circle collision force with the specified radius.
    If radius is not specified, it defaults to the constant one for all nodes.

    Documentation:
        - https://github.com/d3/d3-force#forceCollide

    :param radius:
    """
    pass

  def radius(self, fnc):
    """
    Description:
    -----------
    If radius is specified, sets the radius accessor to the specified number or function,
    re-evaluates the radius accessor for each node, and returns this force.
    If radius is not specified, returns the current radius accessor

    Related Pages:

      https://github.com/d3/d3-force#forceCollide

    Attributes:
    ----------
    :param fnc:
    """
    pass

  def strength(self, strength):
    """
    Description:
    -----------
    If strength is specified, sets the force strength to the specified number in the range [0,1] and returns this force.
    If strength is not specified, returns the current strength which defaults to 0.

    Related Pages:

      https://github.com/d3/d3-force#forceCollide

    Attributes:
    ----------
    :param strength:
    """
    pass

  def iterations(self, iterations):
    """
    Description:
    -----------
    If iterations is specified, sets the number of iterations per application to the specified number and returns this force

    Related Pages:

      https://github.com/d3/d3-force#forceCollide

    Attributes:
    ----------
    :param iterations:
    """
    pass


class D3ForceCenter:
  def __init__(self):
    pass

  def x(self):
    pass

  def y(self):
    pass


class D3Event:
  def __init__(self):
    """

    Documentation:
      - https://github.com/d3/d3-selection/blob/v1.4.0/README.md#event

    """
    pass

  def on(self):
    pass

  @property
  def active(self):
    pass

  @property
  def x(self):
    pass

  @property
  def y(self):
    pass

  def timeStamp(self):
    pass


class D3Pack:
  def __init__(self):
    """

    Documentation:
      - https://github.com/d3/d3-hierarchy/blob/v1.1.8/README.md#pack

    """

  def radius(self, number):
    pass

  def size(self, number):
    """
    Description:
    -----------
    If size is specified, sets this pack layout’s size to the specified two-element array of numbers [width, height]
    and returns this pack layout. If size is not specified, returns the current size, which defaults to [1, 1]

    Related Pages:

      https://github.com/d3/d3-hierarchy/blob/v1.1.8/README.md#pack

    Attributes:
    ----------
    :param number:
    """
    pass

  def padding(self, number):
    """
    If padding is specified, sets this pack layout’s padding accessor to the specified number or function and returns this pack layout. If padding is not specified, returns the current padding accessor, which defaults to the constant zero

    Documentation:
      - https://github.com/d3/d3-hierarchy/blob/v1.1.8/README.md#pack

    :param number:
    :return:
    """
    pass

  def packSiblings(self, circles):
    """
    Packs the specified array of circles, each of which must have a circle.r property specifying the circle’s radius

    Documentation:
      - https://github.com/d3/d3-hierarchy/blob/v1.1.8/README.md#pack

    :param circles:
    :return:
    """
    pass

  def packEnclose(self, circles):
    """
    Computes the smallest circle that encloses the specified array of circles, each of which must have a circle.r property specifying the circle’s radius, and circle.x and circle.y properties specifying the circle’s center

    Documentation:
      - https://github.com/d3/d3-hierarchy/blob/v1.1.8/README.md#pack

    :param circles:
    :return:
    """
    pass


class D3Band:
  """
  https://github.com/d3/d3-scale/blob/master/README.md#band-scales
  """

  def __init__(self, selector):
    """

    """
    self._selector = selector
    self._js = []

  def domain(self, domain=None):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param domain:
    """
    if domain is None:
      self._js.append("domain()")
    return self

  def range(self, range=None):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param range:
    """

  def rangeRound(self, range):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param range:
    """

  def round(self):
    """

    :return:
    """

  def toStr(self):
    """
    Javascript representation

    :return: Return the Javascript String
    """
    if self._selector is None:
      raise Exception("Selector not defined, use this() or new() first")

    if len(self._js) == 0:
      return self._selector

    strData = "%(jqId)s.%(items)s" % {'jqId': self._selector, 'items': ".".join(self._js)}
    self._js = [] # empty the stack
    return strData


class D3File:

  def __init__(self, src, filename, selector):
    self._f, self.src, self._selector = filename, src, selector
    self._js_frg, self._js_ids, self._js_thens, self.profile = [], set(), [], None

  def records(self, jsId):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param jsId: String. The variable id to store the records.
    """
    self._js_frg.append("%s.push(row)" % jsId)
    return self

  def unpack(self, jsId, column=None):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param jsId: String. The variable id to store the records.
    :param column: String | Js Object. The column name.
    """
    if jsId not in self._js_ids:
      column = JsUtils.jsConvertData(column, None)
      self._js_frg.append("%s.push(row[%s])" % (jsId, column))
      self._js_ids.add(jsId)
    return JsObject.JsObject(jsId, isPyData=False)

  def filter(self, rules, keep=True):
    """
    Description:
    -----------
    Add a filter on a row. The rule needs to be fully defined.

    Usage::

      csv.filter("row['direction'] != 'Decreasing'")

    Attributes:
    ----------
    :param rules:
    :param keep: Boolean. Optional.
    """
    if not isinstance(rules, list):
      rules = [rules]
    if not keep:
      self._js_frg.append("if(%s){ return; }" % "&&".join(rules))
    else:
      self._js_frg.append("if(!(%s)){ return; }" % "&&".join(rules))
    return self

  def filterCol(self, column, value, operator="==", keep=True):
    """
    Description:
    -----------
    Add a filter on a specific column. Those functions can be chained.

    Usage::

      text = page.ui.input("Italy")
        page.ui.button("Click").click([
          page.js.d3.csv(data_urls.DEMO_COUNTRY).filterCol("Country Name", text.dom.content).filterCol("Year", "2000", ">").row(
            page.js.console.log("row", skip_data_convert=True)).get(page.js.console.log("data", skip_data_convert=True))
        ])

    Attributes:
    ----------
    :param column: String | JsObject. The column name.
    :param value: String | JsObject. The column value.
    :param operator: String. Optional. The comparison operator.
    :param keep: Boolean. Optional.
    """
    column = JsUtils.jsConvertData(column, None)
    value = JsUtils.jsConvertData(value, None)
    if not keep:
      self._js_frg.append("if(row[%s] %s %s){ return; }" % (column, operator, value))
    else:
      self._js_frg.append("if(!(row[%s] %s %s)){ return; }" % (column, operator, value))
    return self

  def callback(self, jsFnc, profile=None):
    """
    Description:
    -----------

    Usage::

    Attributes:
    ----------
    :param jsFnc: String. Javascript function.
    :param profile: Boolean | Dictionary. Optional. A flag to set the component performance storage.
    """
    self._js_frg.append(jsFnc)
    self.profile = profile
    return self

  def then(self, jsFncs, profile=False):
    """
    Description:
    -----------
    As the file loading with D3 is a promise, it is possible to put events when the response is received.
    This could allow the loading of components.

    Add a post process after on the entire data.

    This function can be chained.

    Usage::

      text = page.ui.input("Italy")
        page.ui.button("Click").click([
          page.js.d3.csv(data_urls.DEMO_COUNTRY).filterCol("Country Name", text.dom.content).filterCol("Year", "2000", ">").row(
            page.js.console.log("row", skip_data_convert=True)).get(page.js.console.log("data", skip_data_convert=True))
        ])

    Attributes:
    ----------
    :param jsFncs: List | String. Javascript functions.
    :param profile: Boolean | Dictionary. Optional. A flag to set the component performance storage.
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._js_thens.append(
      "then(function(data) {%s; return data})" % JsUtils.jsConvertFncs(jsFncs, toStr=True, profile=profile))
    return self

  def cast(self, columns, to="float", profile=None):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param columns:
    :param to:
    :param profile:
    """
    for column in columns:
      if to == "float":
        self._js_frg.append(JsUtils.jsConvertFncs(
          "row['%(col)s'] = parseFloat(row['%(col)s'])" % {"col": column}, toStr=True, profile=profile))
      elif to == "int":
        self._js_frg.append(JsUtils.jsConvertFncs(
          "row['%(col)s'] = parseInt(row['%(col)s'])" % {"col": column}, toStr=True, profile=profile))
    return self

  def row(self, jsFncs, profile=False):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param jsFncs: List | String. Javascript functions.
    :param profile: Boolean | Dictionary. Optional. A flag to set the component performance storage.
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._js_frg.append(JsUtils.jsConvertFncs(jsFncs, toStr=True, profile=profile))
    return self

  def get(self, jsFncs, profile=None):
    """
    Description:
    -----------

    Related Pages:

      https://github.com/d3/d3-request

    Usage::

      text = page.ui.input("Italy")
      page.ui.button("Click").click([
        page.js.d3.csv(data_urls.DEMO_COUNTRY).filterCol("Country Name", text.dom.content).filterCol("Year", "2000", ">").row(
          page.js.console.log("row", skip_data_convert=True)).get(page.js.console.log("data", skip_data_convert=True))
      ])

    Attributes:
    ----------
    :param jsFncs: List | String. Javascript functions.
    :param profile: Boolean | Dictionary. Optional. A flag to set the component performance storage.
    """
    return self.then(jsFncs, profile)

  def toStr(self):
    file = JsUtils.jsConvertData(self._f, None)
    if self._js_ids:
      if self._js_thens:
        return "%s; %s(%s, function(row, err){%s; return row}).%s" % (
          ";".join(["var %s = []" % i for i in self._js_ids]), self._selector, file,
          JsUtils.jsConvertFncs(self._js_frg, toStr=True, profile=self.profile), ".".join(self._js_thens))

      return "%s; %s(%s, function(row, err){%s; return row})" % (
        ";".join(["var %s = []" % i for i in self._js_ids]), self._selector, file,
        JsUtils.jsConvertFncs(self._js_frg, toStr=True, profile=self.profile))

    if self._js_thens:
      return "%s(%s, function(row, err){%s; return row}).%s" % (
        self._selector, file, JsUtils.jsConvertFncs(self._js_frg, toStr=True, profile=self.profile), ".".join(self._js_thens))

    return "%s(%s, function(row, err){%s; return row})" % (
      self._selector, file, JsUtils.jsConvertFncs(self._js_frg, toStr=True, profile=self.profile))


class D3Svg:
  def __init__(self, src, selector, varName=None, setVar=None, parent=None):
    self.src, self._selector, self.varName = src, selector, varName
    self._js, self.setVar, self.component = [], setVar, parent

  def line(self):
    self._js.append("line()")
    return self

  def x(self):
    self._js.append("x(function(d) { return x(d.date1); })")
    return self

  def y(self):
    self._js.append("y(function(d) { return x(d.date1); })")
    return self

  def selectAll(self, tag):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param tag:
    """
    self._js.append("selectAll(%s)" % JsUtils.jsConvertData(tag, None))
    return D3Select(self.src, selector=self.toStr(), setVar=self.setVar, parent=self.component)

  def toStr(self):
    content = [self._selector] + self._js
    if self.varName is not None and self.setVar:
      return "var %s = %s" % (self.varName, ".".join(content))

    return ".".join(content)


class D3Request(JsPackage):

  def header(self, key, value):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param key:
    :param value:
    """
    key = JsUtils.jsConvertData(key, None)
    value = JsUtils.jsConvertData(value, None)
    return self.fnc('header(%s, %s)' % (key, value))

  def timeout(self, value):
    """
    Description:
    -----------
    If timeout is specified, sets the timeout attribute of the request to the specified number of milliseconds and
    returns this request instance. If timeout is not specified, returns the current response timeout,
    which defaults to 0.

    Related Pages:

      https://github.com/d3/d3-request

    Attributes:
    ----------
    :param value:
    """
    value = JsUtils.jsConvertData(value, None)
    return self.fnc('timeout(%s)' % value)

  def mimeType(self, mine_type):
    """
    Description:
    -----------
    If type is specified, sets the request mime type to the specified value and returns this request instance.
    If type is null, clears the current mime type (if any) instead.
    If type is not specified, returns the current mime type, which defaults to null.
    The mime type is used to both set the "Accept" request header and for overrideMimeType, where supported.

    Related Pages:

      https://www.tutorialsteacher.com/d3js/loading-data-from-file-in-d3js#d3.tsv
      https://github.com/d3/d3-request

    Attributes:
    ----------
    :param mine_type:
    """
    mine_type = JsUtils.jsConvertData(mine_type, None)
    return self.fnc('mimeType(%s)' % mine_type)

  def response(self, jsFncs, profile=False):
    """
    Description:
    -----------

    The response text object is in the variable data.

    Related Pages:

      https://www.tutorialsteacher.com/d3js/loading-data-from-file-in-d3js#d3.tsv

    Attributes:
    ----------
    :param jsFncs:
    :param profile: Boolean | Dictionary. Optional. A flag to set the component performance storage.
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    return self.fnc(
      'response(function(xhr) {var data = xhr.responseTex; %s})' % JsUtils.jsConvertFncs(
        jsFncs, toStr=True, profile=profile))

  def get(self, jsFncs, profile=False):
    """
    Description:
    -----------

    The function parameters is defined with tee variable data

    Related Pages:

      https://www.tutorialsteacher.com/d3js/loading-data-from-file-in-d3js#d3.tsv

    Attributes:
    ----------
    :param jsFncs:
    :param profile: Boolean | Dictionary. Optional. A flag to set the component performance storage.
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    return self.fnc('get(function(data) {%s})' % JsUtils.jsConvertFncs(jsFncs, toStr=True, profile=profile))


class D3GeoProjection(JsPackage):
  lib_alias = {"js": 'd3'}

  def center(self, lat, long):
    """
    Description:
    -----------
    """
    return self.fnc("center([%s, %s])" % (lat, long))

  def scale(self, num):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param num:
    """
    return self.fnc("scale(%s)" % num)

  def rotate(self):
    pass

  def translate(self):
    pass


class D3GeoPath(JsPackage):
  lib_alias = {"js": 'd3'}

  def projection(self, proj):
    pass


class D3Geo:

  def mercator(self):
    pass

  def equirectangular(self):
    pass

  def path(self):
    return D3GeoPath()


class JsD3(JsPackage):

  lib_alias = {"js": 'd3'}
  lib_selector = 'd3'

  @property
  def svg(self):
    return D3Svg(self.src, selector="%s.svg" % self._selector)

  @JsUtils.fromVersion({"d3": "4.0.0"})
  def csv(self, url):
    """
    Description:
    -----------
    Sends http request to the specified url to load .csv file or data and executes callback function with parsed
    csv data objects.

    Related Pages:

      https://www.tutorialsteacher.com/d3js/loading-data-from-file-in-d3js#d3.csv

    Attributes:
    ----------
    :param url: String. The url path of the file.
    """
    return D3File(self.src, url, selector="%s.csv" % self._selector)

  @JsUtils.fromVersion({"d3": "4.0.0"})
  def tsv(self, url):
    """
    Description:
    -----------
    Sends http request to the specified url to load a .tsv file or data and executes callback function with parsed
    tsv data objects.

    Related Pages:

      https://www.tutorialsteacher.com/d3js/loading-data-from-file-in-d3js#d3.tsv

    Attributes:
    ----------
    :param url: String. The url path of the file.
    """
    return D3File(self.src, url, selector="%s.tsv" % self._selector)

  @JsUtils.fromVersion({"d3": "4.0.0"})
  def xml(self, url):
    """
    Description:
    -----------
    Sends http request to the specified url to load an .xml file or data and executes callback function with parsed
    xml data objects.

    Related Pages:

      https://www.tutorialsteacher.com/d3js/loading-data-from-file-in-d3js#d3.xml
      https://github.com/d3/d3-fetch

    Attributes:
    ----------
    :param url: String. The url path of the file.
    """
    return D3File(self.src, url, selector="%s.tsv" % self._selector)

  @JsUtils.fromVersion({"d3": "4.0.0"})
  def json(self, url):
    """
    Description:
    -----------
    Sends http request to the specified url to load .json file or data and executes callback function with parsed
    json data objects.

    Related Pages:

      https://www.tutorialsteacher.com/d3js/loading-data-from-file-in-d3js#d3.json

    Attributes:
    ----------
    :param url: String. The url path of the file.
    """
    return D3File(self.src, url, selector="%s.json" % self._selector)

  def text(self, url):
    """
    Description:
    -----------
    Returns a new request to get the text file at the specified url with the default mime type text/plain.
    If no callback is specified, this is equivalent to:

    Related Pages:

        https://github.com/d3/d3-fetch

    Attributes:
    ----------
    :param url: String. The url path of the file.
    """
    return D3File(self.src, url, selector="%s.text" % self._selector)

  def blob(self, url):
    """
    Description:
    -----------
    Fetches the binary file at the specified input URL as a Blob.
    If init is specified, it is passed along to the underlying call to fetch; see RequestInit for allowed fields.

    Related Pages:

        https://github.com/d3/d3-fetch

    Attributes:
    ----------
    :param url: String. The url path of the file.
    """
    return D3File(self.src, url, selector="%s.blob" % self._selector)

  def html(self, url):
    """
    Description:
    -----------
    Fetches the file at the specified input URL as text and then parses it as HTML.
    f init is specified, it is passed along to the underlying call to fetch; see RequestInit for allowed fields.

    Related Pages:

        https://github.com/d3/d3-fetch

    Attributes:
    ----------
    :param url: String. The url path of the file.
    """
    return D3File(self.src, url, selector="%s.html" % self._selector)

  def image(self, url):
    """
    Description:
    -----------
    Fetches the image at the specified input URL.
    If init is specified, sets any additional properties on the image before loading.

    Related Pages:

        https://github.com/d3/d3-fetch

    Attributes:
    ----------
    :param url: String. The url path of the file.
    """
    return D3File(self.src, url, selector="%s.image" % self._selector)

  def dsv(self, url):
    """
    Description:
    -----------
    Fetches the DSV file at the specified input URL.
    If init is specified, it is passed along to the underlying call to fetch; see RequestInit for allowed fields.
    An optional row conversion function may be specified to map and filter row objects to a more-specific
    representation; see dsv.parse for details.

    Related Pages:

        https://github.com/d3/d3-fetch

    Attributes:
    ----------
    :param url: String. The url path of the file.
    """
    return D3File(self.src, url, selector="%s.tsv" % self._selector)

  def min(self, dataset, jsFnc):
    return JsNumber.JsNumber("d3.min(%s, %s)" % (dataset, jsFnc))

  def max(self, dataset, jsFnc):
    return JsNumber.JsNumber("d3.max(%s, %s)" % (dataset, jsFnc))

  def select(self, id, varName=None):
    if varName is not None:
      return D3Select(self.src, selector="d3.select('%s')" % id, varName=varName, setVar=True)

    return D3Select(self.src, selector="d3.select('%s')" % id, setVar=False)

  def selectAll(self, d3Type, setVar=False):
    return D3Select(d3Type=d3Type, setVar=setVar)

  def scaleLinear(self, range):
    return D3ScaleLinear(range)

  def scaleBand(self, range=None):
    """
    Description:
    -----------
    Constructs a new band scale with the specified domain and range, no padding, no rounding and center alignment.
    If domain is not specified, it defaults to the empty domain.
    If range is not specified, it defaults to the unit range [0, 1].

    Attributes:
    ----------
    :param range:

    :rtype: D3Band
    """
    if range is None:
      self._js.append("scaleBand()")
    return D3Band(self.toStr())

  def forceSimulation(self, nodes=None):
    return D3ForceSimulation(nodes)

  def forceManyBody(self):
    return D3ForceManyBody()

  def scaleOrdinal(self, colors):
    pass

  def forceX(self):
    pass

  def forceY(self):
    pass

  def drag(self):
    pass

  def toStr(self):
    """
    Javascript representation

    :return: Return the Javascript String
    """
    if self._selector is None:
      raise Exception("Selector not defined, use this() or new() first")

    if len(self._js) == 0:
      return self._selector

    strData = "%(jqId)s.%(items)s" % {'jqId': self._selector, 'items': ".".join(self._js)}
    self._js = [] # empty the stack
    return strData
